#!/usr/bin/env python3

import logging
import subprocess
import re
import time
from _gmutils.launcher import Launcher


class XRandr(Launcher):
    """ Взаимодействие с xrandr """
    def __init__(self):
        super(XRandr, self).__init__('xrandr')

    def getAllOutputs(self):
        """
        Все подключенные мониторы
        """
        out = self.run(["-q"])
        return re.findall('^(.+)\sconnected', out, re.M)

    def getPrimaryOutput(self):
        """
        Возвращает имя primary монитора или первого, если нет primary
        """
        out = self.run(["-q"])
        outputs = re.findall('^(.+)\sprimary', out, re.M)
        if len(outputs):  # есть primary-монитор
            output = (outputs[0].split(' '))
            return output[0] if len(output) else ''

        # нет primary-монитора - возвращаем первый попавшийся
        outputs = self.getAllOutputs()
        return outputs[0] if len(outputs) else ''

    def query(self):
        """
        Вывод информации о мониторах в консоль
        """
        pgrep = subprocess.Popen(['pgrep', 'X'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
        out, err = pgrep.communicate()
        if pgrep.returncode != 0:
            # если X сервер не запущен, запустим сами
            xServer = subprocess.Popen(['sudo', '-u', 'autologon', 'X'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
            time.sleep(1)
            if xServer.poll() is not None:
                out, err = xServer.communicate()
                print("Ошибка старта X сервера:")
                print(err)
                exit(1)
            print(self.run(["-q"]))
            xServer.terminate()
        else:
            print(self.run(["-q"]))

    def getModes(self):
        """
        Функция получения списка режимов, поддерживаемых одновременно всеми интерфейсами (пересечение множеств)
        """
        ifaces = self.getInterfacesModes()

        interfaces = list()
        modes = set()
        for key in ifaces:
            if key == 'default':
                interfaces.append('default_')
            else:
                interfaces.append(key)
            if not modes:
                modes = set(ifaces[key])
            else:
                modes &= set(ifaces[key])
        return modes, interfaces

    def getInterfaces(self):
        """ Получение подключенных интерфейсов и их параметров в виде словаря, вида
        {
          'VGA-1': {
            '800x600': {
              'rates': ['60.00', '58.96']
            },
            '1024x768': {
              'rates': ['60.00']
            }
          },
          'HDMI-1': {
            '1024x768': {
              'rates': ['58.77']
            }
          }
        }
        """
        out = self.run(['-q']).split('\n')
        ifaces = {}
        iface = ''
        for line in out:
            data = re.findall(r"^(.+)\sconnected", line)
            if data:
                iface = data[0]  # в строке может быть не более одного интерфейса
                ifaces[iface] = {}
            else:  # если в строке нет интерфейса, значит это параметры текущего
                foundModes = re.findall(r'   (\d+x\d+)', line)  # поиск разрешения монитора
                if foundModes:  # если разрешение найдено, то можно искать и частоты обновления экранов в той же строке с разрешением
                    ifaces[iface][foundModes[0]] = {'rates': [rate for rate in re.findall(r'\d+\.\d+', line) if rate]}
        return ifaces

    def getInterfacesModes(self):
        """
        Функция получения всех подключеных интерфейсов и их режимов
        """
        out = self.run(['-q'])
        out = re.findall(r".*\n", out)

        ifaces = {}
        iface = ""
        for o in out:
            t = re.findall(r"^(.+)\sconnected", o)
            if t:
                iface = t[0]
                ifaces[t[0]] = []
            else:
                t = re.findall(r"   (\d+x\d+)", o)
                if t:
                    ifaces[iface].append(t[0])
        return ifaces

    def getInterfaceGeometry(self, interface):
        out = self.run(['-q'])
        outputs = re.findall('^%s.*\s(\d+)x(\d+)\+(\d+)\+(\d+)' % interface, out, re.M)
        if outputs:
            return {'width': outputs[0][0], 'height': outputs[0][1], 'x': outputs[0][2], 'y': outputs[0][3]}
        return {}

    def run(self, args, autologon=True, pipe=True):
        """
        Запуск XRandr c параметрами
        """
        p = super(XRandr, self).run(args)
        out, err = p.communicate()
        if p.returncode != 0:
            logging.error(err)
            raise Exception("Ошибка выполнения xrandr %s:" % err)
        return out
